# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# ======================================================================

__version__= '3.4'

from Globals import HTML, HTMLFile, DTMLFile, MessageDialog, Persistent
from ImageFile import ImageFile  
import OFS,Acquisition, AccessControl.Role
import Globals, sys, traceback
from OFS.DTMLDocument import DTMLDocument
from AccessControl import getSecurityManager


from OFS.Document import Document

import os,time,string
from Products.ZSQLMethods.SQL import SQL

from OFS.History import Historical, html_diff
from OFS.SimpleItem import SimpleItem
from OFS.Cache import Cacheable












class SQLFormsReporterDocument(OFS.SimpleItem.Item,DTMLDocument):
    """SQL Forms object."""    

    meta_type = 'SQL Form Reporter'
    icon = 'misc_/SQLFormsReporter/icon'
    field_type = ['default','text','password','textarea','select','list','date']
    input_cond = ['LIKE','%LIKE%','=','<','>','<>']
    date_format_dict={
                      1:['%d/%m/%Y',' - dd/mm/yyyy'],
                      2:['%m/%d/%Y',' - mm/dd/yyyy'],
                      3:['%Y/%m/%d',' - yyyy/mm/dd'],
                      4:['%Y/%d/%m',' - yyyy/dd/mm'],
		      5:['%Y/%m/%d-%H:%M:%S',""],
                      10:['%A, %B %d, %Y',""],
                      11:['%a, %b %d, %Y',""],
                      12:['%d %B, %Y',""],
                      13:['%d %b, %Y',""],
                      }


    def index_html(self):
        """ default html file returned when view object """
        return "nothing here"


    manage_input = HTMLFile('dtml/manage_input', globals())
    manage_select = HTMLFile('dtml/manage_select', globals())
    manage_where = HTMLFile('dtml/manage_where', globals())
    
    manage_opt = HTMLFile('dtml/manage_options', globals())
    manage_results = HTMLFile('dtml/manage_results', globals())
    
    datasel = HTMLFile('dtml/datasel', globals())

    errorinternalmethod = HTMLFile('dtml/error', globals())    
    

    #manage tabs
    manage_options=(
                    {'label':'Input','action':'manage_input','help':('SQLFormsReporter','input.stx')},
                    {'label':'Select','action':'manage_select','help':('SQLFormsReporter','select.stx')},
                    {'label':'Where','action':'manage_where','help':('SQLFormsReporter','where.stx')},
                    {'label':'Options','action':'manage_opt'},
                    {'label':'Results','action':'manage_results'},
                    #{'label':'Description', 'action':'manage_main'},
                    #{'label':'Properties', 'action':'manage_propertiesForm'},
                    #{'label':'View', 'action':''},
                    #{'label':'Proxy', 'action':'manage_proxyForm', 'help': ('OFSP','DTML-DocumentOrMethod_Proxy.stx')},
                    
                   ) + Historical.manage_options + SimpleItem.manage_options + Cacheable.manage_options
                   




    


    def __init__(self,id,title,database):
        """ init method """
        self.id = id
        self.globals = ''
        self._vars = ''
        self.input_fields = []     #SQL query parameters (is a part of conditions)
        self.select_fields = []    #what to see on report
        self.where_cond = []       #conditions
        self.manage_opt_action(title,database)
        self.rebuild_SQL()
        self.rebuild_methods()
        self.raw="""
Type your comments about this reporter here.
Type description of this reporter here.

Here is the body of this document.
"""


    def manage_order_fields(self,up='',down='',REQUEST=None):
        """ Change order of fields """
        L=self.select_fields
        if up!='':
            up=int(up)
            if up>0:
                tmp=L[up]
                del L[up]
                L.insert(up-1,tmp)
                del tmp
        if down!='':
            down=int(down)
            if down<(len(L)-1):
                tmp=L[down]
                del L[down]
                L.insert(down+1,tmp)
                del tmp
        self.select_fields=L
        del L
        self.rebuild_SQL()
        self.rebuild_methods()
        if REQUEST is not None:
            return self.manage_select(self,REQUEST)



    
    def manage_input_action(self,ix=[],act='',s_table=[],s_field=[],f_type=[],cond_type=[],opt_or=[],label=[],options=[],REQUEST=None):
        """ Add/Update/Delete self.input_fields """
        if act == 'Delete':
            L=self.input_fields
            ix.reverse()
            for i in ix: del L[i]
            self.input_fields = L
        else:
            L=[]
            for i in range(len(s_table)):
                if str(s_table[i])!='0':
                    if i in opt_or:   val_or=1
                    else:             val_or=0
                    fieldtype = f_type[i]
                    fieldopts = options[i]
                    fieldlabel = label[i]
                    opt_type='xx'
                    for j in getattr(self,s_table[i]).fields():
                        if j['fieldname'] == s_field[i]:
                            opt_type=j['options']
                    
                    if fieldtype == 'default':
                        for j in getattr(self,s_table[i]).fields():
                            if j['fieldname'] == s_field[i]:
                                fieldtype = j['form_type']
                                fieldopts = j['expr']
                                fieldlabel = j['fieldlabel']
                                break
                    if fieldtype not in self.field_type:
                        fieldlabel = fieldopts = '-- not supported --'
                        
                    L.append({'s_table':s_table[i],'s_field':s_field[i],'f_type':fieldtype,'cond_type':cond_type[i],'opt_or':val_or,'label':fieldlabel,'opt_type':opt_type,'options':fieldopts})
            self.input_fields=L
        self.rebuild_SQL()
        self.rebuild_methods()
        if REQUEST is not None:
            return self.manage_input(self,REQUEST)





    def manage_select_action(self,ix=[],act='',s_table=[],s_field=[],j_table=[],j_field=[],visible=[],as=[],f_type=[],label=[],REQUEST=None):
        """ Add/Update/Delete select_fields """
        if act == 'Delete':
            L=self.select_fields
            ix.reverse()
            for i in ix: del L[i]
            self.select_fields = L
        else:
            L=[]
            expr=1
            for i in range(len(s_table)):
		if str(s_table[i])=='0': continue #ignore when no table selected
                if i in visible:    is_visible = 1
                else:   is_visible = 0
                if as[i]=='':
                    while "expr"+str(expr) in as:
                        expr=expr+1
                    val_as="expr"+str(expr)
                    expr=expr+1
                    
                else: val_as=as[i]
		fieldtype = f_type[i]
		if fieldtype == 'default':
                        for j in getattr(self,s_table[i]).fields():
                            if j['fieldname'] == s_field[i]:
                                fieldtype = j['form_type']
                                fieldopts = j['expr']
                                fieldlabel = j['fieldlabel']
                                break
                if fieldtype not in self.field_type:
                        fieldlabel = fieldopts = '-- not supported --'
		
                if str(s_table[i]) != '0':
                    fieldlabel=label[i]
                    try:
                        if fieldlabel=="":
                            for j in getattr(self,s_table[i]).fields():
                                    if j['fieldname'] == s_field[i]:
                                        fieldlabel = j['fieldlabel']
                                        break
                    except:
                        pass
                    L.append({'s_table':s_table[i],'s_field':s_field[i],'j_table':j_table[i],'j_field':j_field[i],'visible':is_visible,'f_type':fieldtype,'as':val_as,'label':fieldlabel})
            self.select_fields=L
            
        #if REQUEST is not None:
        #    return Globals.MessageDialog( title='Edited', message='<strong>%s</strong> has been edited.' % self.id, action ='manage_select',)
        self.rebuild_SQL()
        self.rebuild_methods()
        if REQUEST is not None:
            return self.manage_select(self,REQUEST)
        




    def manage_where_action(self,ix=[],act='',opt_or=[],s_table=[],s_field=[],cond_type=[],expr=[],REQUEST=None):
        """ Add/Update/Delete self.where_cond """
        if act == 'Delete':
            L=self.where_cond
            ix.reverse()
            for i in ix: del L[i]
            self.where_cond = L
        else:
            L=[]
            for i in range(len(s_table)):
                if i in opt_or:   val_or=1
                else:             val_or=0
                if str(s_table[i]) != '0':
                    L.append({'opt_or':val_or,'s_table':s_table[i],'s_field':s_field[i],'cond_type':cond_type[i],'expr':expr[i]})
            self.where_cond=L
        self.rebuild_SQL()
        self.rebuild_methods()
        if REQUEST is not None:
            return self.manage_where(self,REQUEST)





    def manage_opt_action(self,title='',database='',security_input='',security_report='',denied_method='',headers='yes',c_title='#eeeeee',c1='#ffeeee',c2='#eeeeff',REQUEST=None,date_format='%Y/%m/%d'):
        """ update options """
        self.title=title
        self.database=database
        self.rebuild_SQL()
        self.security_input=security_input
        self.security_report=security_report
        self.denied_method=denied_method
        self.headers=headers
        self.c_title=c_title
        self.c1=c1
        self.c2=c2
        
        self.date_format=date_format

        self.rebuild_methods()
        self.rebuild_SQL()
        self.SQL = SQL('SQL', 'Internal generic SQL Method', self.database, 'q','<dtml-var q>')
        if REQUEST is not None:
            return Globals.MessageDialog( title='Edited', message='<strong>%s</strong> has been edited.' % self.id, action ='manage_opt',)
        


    def rebuild_SQL(self):
        """ reconstruct internal SQL method """
        #build parameters
        params=''
        for i in self.input_fields:
            try:
                fieldlabel=remove_invalid_chars(i['label'])
                params=params+getattr(self,i['s_table']).sq_table+'_'+i['s_field']+'_'+fieldlabel+'''
'''
            except:
                self.PrintError("Can't update SQL method parameters")
            
        
        #build body
        body='''
<dtml-comment>
   This is an automatically generated SQL method!
   Do not edit!
</dtml-comment>

SELECT'''
        inner_list=[]     #INNER JOIN special list
        show_tables=[]    #list of tables used to show info and are not into INNER JOIN
        inner_tables=[]   #list of tables which are  used into INNER JOIN

        ########################################
        #                                      #
        #build SELECT xx.xx as xx              #
        #                                      #
        ########################################
        
        for i in self.select_fields:
            try:
                if int(i['visible'])!=0:
                    T1=getattr(self,i['s_table']).sq_table
                    body=body+'''
   %s.%s as %s ,''' % (T1,i['s_field'],i['as'],)
            except:
                self.PrintError("Can't update SQL method -> SELECT area")
                
        body=body[:-1]+'''
FROM        
'''

        #build inner_table
        for i in self.select_fields:
            try:
                if str(i['j_table']) != '0':
                    T1=getattr(self,i['s_table']).sq_table
                    F1=i['s_field']
                    T2=getattr(self,i['j_table']).sq_table
                    F2=i['j_field']
                    inner_list.append({'T1':T1,'F1':F1,'T2':T2,'F2':F2})
                    #add into inner_tables ........ T1 & T2
                    if T1 not in inner_tables:
                        inner_tables.append(T1)
                    if T2 not in inner_tables:
                        inner_tables.append(T2)
            except:
                self.PrintError("Can't update SQL method -> FROM area")

                
        #build show_tables
        for i in self.select_fields:
            try:
                if int(i['visible'])!=0:
                    T1=getattr(self,i['s_table']).sq_table
                    if T1 not in show_tables:
                        if T1 not in inner_tables:
                            show_tables.append(T1)
            except:
                self.PrintError("Can't update SQL method -> FROM area")
        for i in self.where_cond:
            try:
                if str(i['s_table']) != '0':
                    T1=getattr(self,i['s_table']).sq_table
                    if T1 not in show_tables and T1 not in inner_tables:
                        show_tables.append(T1)
            except:
                self.PrintError("Can't update SQL method -> FROM area")
        for i in self.input_fields:
            try:
                if str(i['s_table']) != '0':
                    T1=getattr(self,i['s_table']).sq_table
                    if T1 not in show_tables and T1 not in inner_tables:
                        show_tables.append(T1)
            except:
                self.PrintError("Can't update SQL method -> FROM area")




        #build list tables which are used but are not in inner-join
        if show_tables != []:
            for i in show_tables:
                body=body+i+","
            body=body[:-1]


        ###########################################
        #                                         #
        #build INNER JOIN                         #
        #                                         #
        ###########################################
        if inner_list != []:
            if show_tables != []:
                body=body+","
            n=len(inner_list)
            cnt=0
            for j in range(0,n):
                    body=body+"("
            for i in inner_list:
                if cnt==0:
                    body=body+"""
  %s INNER JOIN %s ON %s.%s = %s.%s)""" % (i['T1'],i['T2'],i['T1'],i['F1'],i['T2'],i['F2'],)
                else:
                    body=body+"""
  INNER JOIN %s ON %s.%s = %s.%s)""" % (i['T2'],i['T1'],i['F1'],i['T2'],i['F2'],)
                cnt=cnt+1
                

        ###########################################
        #                                         #
        #build WHERE                              #
        #                                         #
        ###########################################
        body=body+'''
'''
        cnt=0
        for i in self.where_cond:
            try:
                if cnt>0:
                    if int(i['opt_or'])==1:  body=body+" OR "
                    else:                     body=body+" AND "
                else: body=body+'''WHERE '''
                T1=getattr(self,i['s_table']).sq_table
                F1=i['s_field']
                sq_type='x'
                for j in getattr(self,i['s_table']).fields():
                        if j['fieldname']==F1:
                            sq_type=j['sq_type']
                            break
                if sq_type in ['integer','float']: xstr=""
                else: xstr="'"
                if i['cond_type'] == '%LIKE%':
                    body=body+"""%s.%s LIKE '%%%s%%'
""" % (getattr(self,i['s_table']).sq_table,i['s_field'],i['expr'])
                else:
                    body=body+"""%s.%s %s %s%s%s
""" % (getattr(self,i['s_table']).sq_table,i['s_field'],i['cond_type'],xstr,i['expr'],xstr)
                    
                
            except:
                self.PrintError("Can't update SQL method -> WHERE area")
                import sys, traceback, string
                type, val, tb = sys.exc_info()
                sys.stderr.write(string.join(traceback.format_exception(type, val, tb), ''))
                del type, val, tb

            cnt=cnt+1
        
        for i in self.input_fields:
            try:
		extralabel=remove_invalid_chars(i['label'])
                if cnt>0:
                    if int(i['opt_or'])==1:  body=body+" OR "
                    else:                     body=body+" AND "
                else: body=body+'''WHERE '''
                if str(i['s_table']) != '0':
                    T1=getattr(self,i['s_table']).sq_table
                    F1=i['s_field']
                    sq_type='x'
                    body=body+"\n"
                    for j in getattr(self,i['s_table']).fields():
                        if j['fieldname']==F1:
                            sq_type=j['sq_type']
                            break
                    if sq_type in ['integer','float'] : xstr=""
                    else: xstr="'"
                    if i['cond_type']=='%LIKE%':
                        if i['f_type']=='date':
                            body=body+"""%s.%s LIKE '%%<dtml-var "_.int(ZopeTime(%s_%s_%s))">%%' """ % (T1,F1,T1,F1,extralabel,)
                        elif i['f_type']=='list':
                            body=body+"""%s.%s LIKE '%%<dtml-var "_.string.join(%s_%s_%s,',')">%%' """ % (T1,F1,T1,F1,extralabel,)
                        else:
                            body=body+"""%s.%s LIKE '%%<dtml-var %s_%s_%s>%%' """ % (T1,F1,T1,F1,extralabel,)
                    else:
                        if i['f_type']=='date':
                            body=body+"""%s.%s %s %s<dtml-var "_.int(ZopeTime(%s_%s_%s))">%s """ % (T1,F1,i['cond_type'],xstr,T1,F1,extralabel,xstr,)
                        elif i['f_type']=='list':
                            body=body+"""%s.%s %s %s<dtml-var "_.string.join(%s_%s_%s,',')">%s """ % (T1,F1,i['cond_type'],xstr,T1,F1,extralabel,xstr,)
                        else:
                            body=body+"""%s.%s %s %s<dtml-var %s_%s_%s>%s """ % (T1,F1,i['cond_type'],xstr,T1,F1,extralabel,xstr,)
            except:
                self.PrintError("Can't update SQL method -> WHERE area")
            cnt=cnt+1

            
            
        self.runSQL = SQL('runSQL', 'Internal SQL Method', self.database, params, body)






    def rebuild_methods(self):
        """ reconstruct internal input & report methods """
        #############################
        #                           #
        #input                      #
        #                           #
        #############################
        if hasattr(self,'input'): del self.input
        if self.input_fields == []:
            body="<dtml-var report>"
        else:
            body=""
            if self.headers=='yes':        body=body+"""<dtml-var standard_html_header>\n<dtml-if %s>\n""" % (self.security_input,)
            body=body+"""
<dtml-unless REFERER><dtml-call "REQUEST.set('REFERER',HTTP_REFERER)"></dtml-unless>
<h1><dtml-var title_or_id></h1>
<script>
function parse_date(s)
 {
 i1=s.indexOf("/");
 i2=i1+(s.slice(i1+1)).indexOf("/")+1;
 if (i1<0 || i2<0)
  {
   year=<dtml-var "ZopeTime().strftime('%Y')">;
   month=<dtml-var "ZopeTime().strftime('%m')">-1;
   day=<dtml-var "ZopeTime().strftime('%d')">;
  }
 else
  {
   year=s.slice(0,i1);
   month=s.slice(i1+1,i2)-1;
   day=s.slice(i2+1);
  }
 return 'datasel?year='+year+'&month='+month+'&day='+day;
 }
</script>
<form action="report" method="post">
<input type="hidden" name="REFERER" value="<dtml-var REFERER>">
<table>
"""
            for i in self.input_fields:
                try:
		    fieldlabel = i['label']
		    fieldname=getattr(self,i['s_table']).sq_table+'_'+i['s_field']+'_'+remove_invalid_chars(fieldlabel)
                    fieldtype = i['f_type']
                    opttype = i['opt_type']
                    fieldsize = 10
                    fieldopts = i['options']
                    if fieldtype == 'default':
                        for j in getattr(self,i['s_table']).fields():
                            if j['fieldname'] == i['s_field']:
                                fieldtype = j['form_type']
                                fieldsize = j['form_size']
                                if fieldopts == "": fieldopts = j['options']
                                if fieldlabel == "": fieldlabel = j['fieldlabel']
                                break
                    body=body+"<tr>\n"
                
                    #text
                    if fieldtype == 'text':
                        body=body+"""  <th align="left">%s</th><td><input type="text" name="%s" value="<dtml-if %s><dtml-var %s><dtml-else>%s</dtml-if>" size="%s"></td>\n""" % (fieldlabel,fieldname,fieldname,fieldname,fieldopts,fieldsize,)
                    #password
                    if fieldtype == 'password':
                        body=body+"""  <th align="left">%s</th><td><input type="password" name="%s" value="<dtml-if %s><dtml-var %s><dtml-else>%s</dtml-if>" size="%s"></td>\n""" % (fieldlabel,fieldname,fieldname,fieldname,fieldopts,fieldsize,)
                    #textarea
                    if fieldtype == 'textarea':
                        body=body+"""  <th align="left">%s</th><td><textarea rows="5" name="%s" cols="%s"><dtml-if %s><dtml-var %s><dtml-else>%s</dtml-if></textarea></td>\n""" % (fieldlabel,fieldname,fieldsize,fieldname,fieldname,fieldopts,)
                    #date
                    if fieldtype == 'date':
                        body=body+"""  <th align="left">%s</th>
 <td>
  <input type="text" name="view_%s" size="%s" value="<dtml-if %s><dtml-var %s><dtml-else>%s</dtml-if>" disabled>
  <input type="hidden" name="%s" value="<dtml-if %s><dtml-var %s><dtml-else>%s</dtml-if>">
  <input type=button value="..." onClick='window.retfield = document.forms[0].view_%s; window.datafield = document.forms[0].%s; window.open(parse_date(window.datafield.value), "chooser", "toolbar=no,menubar=no,scrollbars=yes,width=300,height=300")'>
 </td>""" % (fieldlabel,
             fieldname,fieldsize,fieldname,fieldname,fieldopts,
             fieldname,fieldname,fieldname,fieldopts,
             fieldname,fieldname)
 
                    #select
                    if fieldtype == 'select':
                        if opttype == 'sqt':
                            #assume there is an SQL table query
                            opts=string.split(fieldopts+',,,,',',')
                            table=opts[0]
                            f_code=opts[1]
                            f_name=opts[2]
                            f_where=opts[3]
                            f_sort=opts[4]
                            if f_where!='':
                                f_where='where '+f_where
                            if f_sort!='':
                                f_sort='order by '+f_sort
                            body=body+"""
  <th align="left">%s</th>
  <td>
   <select name="%s">
     <dtml-in "SQL(q='select %s as SQLFORM_CODE,%s as SQLFORM_NAME from %s %s %s')">
       <option value="<dtml-var SQLFORM_CODE>" <dtml-if %s><dtml-if "_.str(%s) == _.str(SQLFORM_CODE)"> SELECTED</dtml-if></dtml-if>><dtml-var SQLFORM_NAME></option>
     </dtml-in>
   </select>
  </td>
""" % (fieldlabel,fieldname,f_code,f_name,table,f_where,f_sort,fieldname,fieldname)
                        else:
                            #assume there is a python dict!
                            body=body+'''
  <th align="left">%s</th>
  <td>
  <dtml-call "REQUEST.set('%s_expr',%s)">
  <dtml-call "REQUEST.set('%s_keys',%s_expr.keys())">
  <dtml-call "%s_keys.sort()">
  <select name="%s">
   <dtml-in "%s_keys">
     <option value="<dtml-var sequence-item>"<dtml-if %s><dtml-if "_.str(%s)==_.str(_['sequence-item'])"> SELECTED</dtml-if></dtml-if>><dtml-var "%s_expr[_['sequence-item']]"></option>
   </dtml-in>
  </select>
  </td>
''' % (fieldlabel,
       fieldname,fieldopts, #REQ set 1
       fieldname,fieldname, #REQ set 2
       fieldname, #sort
       fieldname,fieldname,fieldname,fieldname,fieldname)

                            

                    #list
                    if fieldtype == 'list':
                        if opttype == 'sqt':
                            #assume there is an SQL table query
                            opts=string.split(fieldopts+',,,,',',')
                            table=opts[0]
                            f_code=opts[1]
                            f_name=opts[2]
                            f_where=opts[3]
                            f_sort=opts[4]
                            if f_where!='':
                                f_where='where '+f_where
                            if f_sort!='':
                                f_sort='order by '+f_sort
                            body=body+"""
  <th align="left" valign="top">%s</th>
  <td>
   <dtml-in "SQL(q='select %s as SQLFORM_CODE,%s as SQLFORM_NAME from %s %s %s')">
     <input type="checkbox" name="%s:list:string" value="<dtml-var SQLFORM_CODE>" <dtml-if %s><dtml-if "_.str(SQLFORM_CODE)==_.str(%s) or _.str(SQLFORM_CODE) in _.string.split(_.str(%s),',')"> CHECKED</dtml-if></dtml-if>><dtml-var SQLFORM_NAME><br>
   </dtml-in>
  </td>
""" % (fieldlabel,f_code,f_name,table,f_where,f_sort,fieldname,fieldname,fieldname,fieldname)

                        else:
                            #assume there is a python dict!
                            body = body + '''
   <th align="left" valign="top">%s</th>
   <td>
   <dtml-call "REQUEST.set('%s_expr',%s)">
   <dtml-call "REQUEST.set('%s_keys',%s_expr.keys())">
   <dtml-call "%s_keys.sort()">
   <dtml-in "%s_keys">
     <input type="checkbox" name="%s:list:string" value="<dtml-var sequence-item>"<dtml-if %s><dtml-if "_.str(%s)==_.str(_['sequence-item']) or _.str(_['sequence-item']) in _.string.split(_.str(%s),',')"> CHECKED</dtml-if></dtml-if>><dtml-var "%s_expr[_['sequence-item']]"><br>
   </dtml-in>
   </td>
 ''' % (fieldlabel,
        fieldname,fieldopts,    #REQ set 1
        fieldname,fieldname,    #REQ set 2
        fieldname, #sort
        fieldname, #dtml-in
        fieldname,fieldname,fieldname,fieldname,fieldname)
 


                
                except:
                    self.PrintError("Can't update SQL method parameters")
            body=body+"""
</table>
 &nbsp;&nbsp;&nbsp;&nbsp;
 <input type="submit" value="Send">
 &nbsp;&nbsp;&nbsp;&nbsp;
 <input type="button" value="Cancel" onclick="JavaScript:location.href='<dtml-var REFERER>';"> 
</form>\n"""
            if self.headers=='yes':        body=body+"""</dtml-if>\n<dtml-var standard_html_footer>"""

        self.input = Document(body, __name__='input')
        self.input.title='Internal input method'




        ###############################
        #                             #
        #report                       #
        #                             #
        ###############################
        if hasattr(self,'report'): del self.report
        body=""
        if self.headers=='yes':     body=body+"""<dtml-var standard_html_header>\n<dtml-if %s>\n""" % (self.security_report,)
        body=body+"""
<h1><dtml-var title_or_id></h1>
<dtml-unless REFERER><dtml-call "REQUEST.set('REFERER',HTTP_REFERER)"></dtml-unless>
<form><input type="button" value="Back" onclick="JavaScript:location.href='<dtml-var REFERER>';"></form>

<table>
<tr BGCOLOR='&dtml-c_title;'>
"""
        for i in self.select_fields:
                if int(i['visible'])!=0:
                    body=body+"  <th>%s</th>\n" % (i['label'])

                    
        body=body+"""</tr>
<dtml-call "REQUEST.set('flag',0)">
<dtml-in runSQL>
 <tr align="left" valign="top" BGCOLOR='<dtml-if "_.int(flag)==1">&dtml-c1;<dtml-else>&dtml-c2;</dtml-if>'>
"""
        for i in self.select_fields:
                if int(i['visible'])!=0:
                    try:
                        fieldname = getattr(self,i['s_table']).sq_table+'_'+i['s_field']
                        if i['s_field'] == getattr(self,i['s_table']).primary_key:
                            is_primary = 1
                        else:
                            is_primary = 0
                        fieldlabel = i['label']
                        for j in getattr(self,i['s_table']).fields():
                                if j['fieldname'] == i['s_field']:
				    ### AICI
                                    #fieldtype = j['form_type']
				    fieldtype = i['f_type']
                                    sqtype = j['sq_type']
                                    options = j['options']
                                    expr = j['expr']
                                    if fieldlabel == "": fieldlabel = j['fieldlabel']
                                    break
                        if fieldtype == 'date' or options == 'cd':
                            if sqtype in ['integer','float']:
                                #is possibile to be a date field stored as number?
                                body = body + """
  <td><dtml-var "_.DateTime(_.int(%s)).strftime('%s')"></td>""" % (i['as'],self.date_format)
                            else:
                                #date field is stored as string
                                body=body+"""\n <td><dtml-var %s></td>""" % (i['as'])

                        elif fieldtype == 'text':
                            if is_primary == 1:
                                body=body+"""  <td><a href="<dtml-var URL2>/%s/edit?edit_key=<dtml-var %s>&REFERER=&dtml-URL2;"><dtml-var %s></a></td>\n""" % (i['s_table'],i['as'],i['as'],)
                            else:
                                body=body+"""  <td><dtml-var %s></td>\n""" % (i['as'],)

                        elif fieldtype == 'password':
                            body=body+"""  <td>***</td>\n"""

                        elif fieldtype == 'textarea':
                            body=body+"""  <td><dtml-var %s newline_to_br></td>\n""" % (i['as'],)

                        elif fieldtype == 'select':
                            if options == 'sqt':
                                #assume there is an SQL table query
                                opts=string.split(expr+',,,,',',')
                                table=opts[0]
                                f_code=opts[1]
                                f_name=opts[2]
                                body=body + """
 <td>
 <dtml-try>
 <dtml-in "SQL(q='select %s as SQLFORM_NAME from %s where %s = '+_.str(%s))">
   <dtml-var SQLFORM_NAME>
 </dtml-in>
 <dtml-except>
  <dtml-var errorinternalmethod>
 </dtml-try>
</td>""" % (f_name,table,f_code,i['as'])
                            else:
                                #assume there is a python dict!
                                body = body + '''
 <td>
<dtml-try>
   <dtml-call "REQUEST.set('%s_expr',  %s  )">
   <dtml-var "%s_expr[_.int(%s)]">
<dtml-except>
  <dtml-var errorinternalmethod>
</dtml-try>
 </td>''' % (i['as'],expr,i['as'],i['as'])

                        elif fieldtype == 'list':
                            if options == 'sqt':
                                #assume there is an SQL table query
                                opts = string.split(expr+',,,,',',')
                                table = opts[0]
                                f_code = opts[1]
                                f_name = opts[2]
                                body = body+"""
 <td>
 <dtml-try>
 <dtml-if "%s!='0'">
 <ul>
 <dtml-in "_.string.split(%s,',')">
   <dtml-in "SQL(q='select %s as SQLFORM_NAME from %s where %s = '+_.str(_['sequence-item']))">
    <li><dtml-var SQLFORM_NAME></li>
   </dtml-in>
 </dtml-in>
 </ul>
 </dtml-if>
 <dtml-except>
    <dtml-var errorinternalmethod>
 </dtml-try>
 </td>
""" % (i['as'],i['as'],f_name,table,f_code)
                            else:
                                #assume there is a python dict!
                                body = body + """
 <td>
 <dtml-try>
   <dtml-if "%s!='0'">
   <dtml-call "REQUEST.set('%s_expr',  %s  )">
   <dtml-call "REQUEST.set('%s_list',_.string.split(%s,','))">
     <ul>
     <dtml-in %s_list>
       <li><dtml-var "%s_expr[_.int(_['sequence-item'])]"></li>
     </dtml-in>
     </ul>
    </dtml-if>
 <dtml-except>
    <dtml-var errorinternalmethod>
 </dtml-try>
 </td>
""" % (
       i['as'],
       i['as'],expr,
       i['as'],i['as'],
       i['as'],i['as'],)

                            
                    except:
                        body=body+"""\n <td>error</td>"""
                        #self.PrintError("Can't update SQL method parameters")

                    
        body=body+"""</tr>
<dtml-call "REQUEST.set('flag',1-_.int(flag))">
</dtml-in>
</table>
<form><input type="button" value="Back" onclick="JavaScript:location.href='<dtml-var REFERER>';"></form>
"""
        if self.headers=='yes':     body=body+"""\n</dtml-if>\n<dtml-var standard_html_footer>"""
        self.report = Document(body, __name__='report')
        self.report.title='Internal report method'
        





    def PrintError(self,s):
	    """ Print an error on server console """
	    print "\n\nSQL Forms Reporter: "+s
	    import sys, traceback, string
	    type, val, tb = sys.exc_info()
	    sys.stderr.write(string.join(traceback.format_exception(type, val, tb), ''))
	    #del type, val, tb
        


def remove_invalid_chars(s):
	t=s.replace(' ','_')
	for i in [':','-','/','\\','*','+',',','"',"'",';']:
	    t=t.replace(i,'')
	return t

Globals.default__class_init__(SQLFormsReporterDocument)

